'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.printFileSizesAfterBuild = printFileSizesAfterBuild;
exports.measureFileSizesBeforeBuild = measureFileSizesBeforeBuild;

var fs = _interopRequireWildcard(require("fs-extra"));

var path = _interopRequireWildcard(require("path"));

var _chalk = _interopRequireDefault(require("chalk"));

var _filesize = _interopRequireDefault(require("filesize"));

var _recursiveReaddir = _interopRequireDefault(require("recursive-readdir"));

var _stripAnsi = _interopRequireDefault(require("strip-ansi"));

var _gzipSize = require("gzip-size");

var logger = _interopRequireWildcard(require("../utils/logger"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function canReadAsset(asset) {
  return /\.(js|css)$/.test(asset) && !/service-worker\.js/.test(asset) && !/precache-manifest\.[0-9a-f]+\.js/.test(asset);
} // Prints a detailed summary of build files.


function printFileSizesAfterBuild(webpackStats, previousSizeMap, buildFolder, maxBundleGzipSize, maxChunkGzipSize) {
  var _webpackStats$assets;

  const root = previousSizeMap.root;
  const sizes = previousSizeMap.sizes;
  const assets = ((_webpackStats$assets = webpackStats.assets) === null || _webpackStats$assets === void 0 ? void 0 : _webpackStats$assets.filter(asset => canReadAsset(asset.name)).map(asset => {
    const fileContents = fs.readFileSync(path.join(root, asset.name));
    const size = (0, _gzipSize.sync)(fileContents);
    const previousSize = sizes[removeFileNameHash(root, asset.name)];
    const difference = getDifferenceLabel(size, previousSize);
    return {
      folder: path.join(path.basename(buildFolder), path.dirname(asset.name)),
      name: path.basename(asset.name),
      size: size,
      sizeLabel: (0, _filesize.default)(size) + (difference ? ' (' + difference + ')' : '')
    };
  }).sort((a, b) => b.size - a.size)) || [];
  const longestSizeLabelLength = Math.max.apply(null, assets.map(a => (0, _stripAnsi.default)(a.sizeLabel).length));
  let suggestBundleSplitting = false;
  assets.forEach(asset => {
    let sizeLabel = asset.sizeLabel;
    let sizeLength = (0, _stripAnsi.default)(sizeLabel).length;

    if (sizeLength < longestSizeLabelLength) {
      let rightPadding = ' '.repeat(longestSizeLabelLength - sizeLength);
      sizeLabel += rightPadding;
    }

    let isMainBundle = asset.name.indexOf('main.') === 0;
    let maxRecommendedSize = isMainBundle ? maxBundleGzipSize : maxChunkGzipSize;
    let isLarge = maxRecommendedSize && asset.size > maxRecommendedSize;

    if (isLarge && path.extname(asset.name) === '.js') {
      suggestBundleSplitting = true;
    }

    logger.log('  ' + (isLarge ? _chalk.default.yellow(sizeLabel) : sizeLabel) + '  ' + _chalk.default.dim(asset.folder + path.sep) + _chalk.default.cyan(asset.name));
  });

  if (suggestBundleSplitting) {
    logger.log();
    logger.log(_chalk.default.yellow('The bundle size is significantly larger than recommended.'));
    logger.log(_chalk.default.yellow('Consider reducing it with code splitting: https://goo.gl/9VhYWB'));
    logger.log(_chalk.default.yellow('You can also analyze the project dependencies: https://goo.gl/LeUzfb'));
  }
}

function removeFileNameHash(buildFolder, fileName) {
  return fileName.replace(buildFolder, '').replace(/\\/g, '/').replace(/\/?(.*)(\.[0-9a-f]+)(\.chunk)?(\.js|\.css)/, (match, p1, p2, p3, p4) => p1 + p4);
} // Input: 1024, 2048
// Output: "(+1 KB)"


function getDifferenceLabel(currentSize, previousSize) {
  let FIFTY_KILOBYTES = 1024 * 50;
  let difference = currentSize - previousSize;
  let fileSize = !Number.isNaN(difference) ? (0, _filesize.default)(difference) : 0;

  if (difference >= FIFTY_KILOBYTES) {
    return _chalk.default.red('+' + fileSize);
  } else if (difference < FIFTY_KILOBYTES && difference > 0) {
    return _chalk.default.yellow('+' + fileSize);
  } else if (difference < 0) {
    return _chalk.default.green(fileSize);
  } else {
    return '';
  }
}

function measureFileSizesBeforeBuild(buildFolder) {
  return new Promise(resolve => {
    (0, _recursiveReaddir.default)(buildFolder, (err, fileNames) => {
      let sizes = {};

      if (!err && fileNames) {
        sizes = fileNames.filter(canReadAsset).reduce((memo, fileName) => {
          let contents = fs.readFileSync(fileName);
          let key = removeFileNameHash(buildFolder, fileName);
          memo[key] = (0, _gzipSize.sync)(contents);
          return memo;
        }, {});
      }

      resolve({
        root: buildFolder,
        sizes: sizes
      });
    });
  });
}